@@ -2,6 +2,7 @@ package t::Helper;
 use Mojo::Base -base;
 
 use JSON::Validator;
+use Mojo::File;
 use Mojo::JSON qw(decode_json encode_json);
 use Mojo::Util qw(monkey_patch);
 use Test::More;
@@ -17,6 +18,7 @@ sub acceptance {
   Test::More::plan(skip_all => $@)                            unless eval "require $schema_class;1";
 
   my $test = sub { +{file => $_[0], group_description => $_[1], test_description => $_[2]} };
+  my $ua   = _acceptance_ua($schema_class);
 
   $acceptance_params{todo_tests} = [map { $test->(@$_) } @{$acceptance_params{todo_tests}}]
     if $acceptance_params{todo_tests};
@@ -30,7 +32,7 @@ sub acceptance {
       my ($schema_p, $data_p) = map { Mojo::JSON::Pointer->new(shift @_) } qw(schema data);
       my ($schema_d, $data_d) = map { clone($_->data) } $schema_p, $data_p;
 
-      my $schema = $schema_class->new($schema_d);
+      my $schema = $schema_class->new($schema_d, ua => $ua);
       return 0 if @{$schema->errors};
 
       my @errors = $schema->validate($data_d);
@@ -38,7 +40,7 @@ sub acceptance {
       # Doing internal tests on mutation, since I think Test::JSON::Schema::Acceptance is a bit too strict
       Test2::Tools::Compare::is(encode_json($data_d),   encode_json($data_p->data),   'data structure is the same');
       Test2::Tools::Compare::is(encode_json($schema_d), encode_json($schema_p->data), 'schema structure is the same')
-        unless _acceptance_schema_contains_invalid_ref($schema_p);
+        unless _skip_schema_is($schema_p);
 
       return @errors ? 0 : 1;
     },
@@ -113,10 +115,61 @@ sub import {
   monkey_patch $caller => validate_ok        => \&validate_ok;
 }
 
-sub _acceptance_schema_contains_invalid_ref {
+sub _acceptance_ua {
+  my $schema_class = shift;
+  require Mojo::UserAgent;
+  require Mojolicious;
+  my $ua  = Mojo::UserAgent->new;
+  my $app = Mojolicious->new;
+
+  $app->static->paths([Mojo::File->new(qw(t spec remotes))->to_string]);
+  $ua->server->app($app);
+
+  $ua->on(
+    $_ => sub {
+      my ($ua, $tx) = @_;
+      my $url = $tx->req->url;
+      $url->scheme(undef)->host(undef)->port(undef) if $url->host and $url->host eq 'localhost';
+    }
+  ) for qw(prepare start);
+
+  my $app_base_url = $ua->get('/')->req->url->to_abs->to_string;
+  $app_base_url =~ s!/$!!;
+
+  my $orig_load_schema = $schema_class->can('_load_schema');
+  monkey_patch $schema_class => _load_schema => sub {
+    my ($self, $url) = @_;
+    my $cached;
+    return $cached, $url if $cached = $self->_store($url);
+    $url =~ s!^https?://localhost:1234!$app_base_url!;
+    return $self->$orig_load_schema($url);
+  };
+
+  #my $orig_resolve_ref = $schema_class->can('_resolve_ref');
+  #monkey_patch $schema_class => _resolve_ref => sub {
+  #  my ($self, $ref_url, $base_url, $schema) = @_;
+  #  $ref_url  =~ s!^https?://localhost:1234!$app_base_url!;
+  #  $base_url =~ s!^https?://localhost:1234!$app_base_url!;
+  #  $self->$orig_resolve_ref($ref_url, $base_url, $schema);
+  #};
+
+  #my $orig_store       = $schema_class->can('_store');
+  #monkey_patch $schema_class => _store => sub {
+  #  my ($self, $id, $schema) = @_;
+  #  $id =~ s!^https?://localhost:1234!$app_base_url!;
+  #  $self->$orig_store($id, $schema);
+  #};
+
+  return $ua;
+}
+
+sub _skip_schema_is {
   my $p     = shift;
   my @paths = ('', '/properties/foo');
 
+  # The URL has been changed by _acceptance_ua()
+  return 1 if encode_json($p->data) =~ m!localhost:1234!;
+
   # JSON::Validator always normalizes $ref with multiple keys
   for my $path (@paths) {
     my $ref = $p->get($path);
